function $feedly( msg  )
{
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
	                     	.getService(Components.interfaces.nsIConsoleService);

	consoleService.logStringMessage( "[feedly]" + msg );
}

var FeedlyExceptionUtils = function()
{
	var that = {};
	
	that.formatError = function  ( task, e ) {
		var m = ["failed to ", task, " because "]
		
		if (e.name || e.message) {
			m.push(e.name)
			m.push(" -- ")
			m.push(e.message)
		} else if (typeof e == "string "){
			m.push ( e )
		} else {
			m.push ( e.toString () )
		}
		
		if (e.fileName || e.lineNumber) {
			m.push(" [")
			if( e.fileName ) {
				m.push(e.fileName)
			}
			if( e.lineNumber ) {
				m.push("@")
				m.push(e.lineNumber)
			}
			m.push("]")
		}
		return m.join("")
	}

	return that;
}();

// Firefox scripts assume that these Cc and Ci are defined.
var Cc = Components.classes;
var Ci = Components.interfaces;

var FEEDLY_UPDATE_HOME = "http://s3.feedly.com/release/";

Components.utils.import( "resource://gre/modules/XPCOMUtils.jsm" );

function FeedlyBootComponent() 
{ 
	this.wrappedJSObject = this;

	// Load JS libraries
	var jsLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);  
	jsLoader.loadSubScript( "chrome://feedly/content/app/scripts/version.js" );

	this.fObservers = [];
	this.bootError = null;
	
	var currentVersion = feedlyApplicationVersion;
	this._startLoadCore();
}

FeedlyBootComponent.prototype = {
	classDescription: 	"Boot system which loads the feedly Application",
 	classID:          	Components.ID("{3301997d-15db-4df3-8581-125b3273c5c9}"),
  	contractID:       	"@devhd.com/feedly-boot;1",
 	QueryInterface: 	XPCOMUtils.generateQI( [ Components.interfaces.nsIFeedlyBoot ] ),	
	
	_startLoadCore : function()
	{
		var phase = "";
		try
		{
			var jsLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);  

			///D $feedly( "[boot][core loader] load include.js...")
			phase = "loading include.js";
			jsLoader.loadSubScript( "chrome://feedly/content/app/core/include.js" );
			
			try
			{
				///D $feedly( "[boot][core loader] declaring feedlyVersion cookie := " + feedlyApplicationVersion );
				this._setCookieString( "http://www.feedly.com", "feedlyVersion=" + feedlyApplicationVersion + "; path=/; domain=.feedly.com" );
			}
			catch( ignore )
			{
			}

			///D $feedly( "[boot][core loader] load feedly core...")
			phase = "loading feedly core";
			loadFeedlyCore();
			
			///D $feedly( "[boot][core loader] notify observers...")
			phase = "notifying observers";
			this.fire( "onCoreLoaded" );
	
			///D $feedly( "[boot][core loader] post processing...")
			phase = "post processing";
			onAfterFeedlyCoreLoaded();			
		}
		catch( e )
		{
			var msg = FeedlyExceptionUtils.formatError( "load core [phase=" + phase + "]", e )
			$feedly( "[boot][start load core][error][phase=" + phase + "]" + msg );

			setBootError( msg );
			return;
		}
	},

	
	_setCookieString : function( uri, cookieString )
	{
		var cookieManager = Components.classes["@mozilla.org/cookiemanager;1"]
									  .getService( Components.interfaces.nsICookieManager );

		var ioService = Components.classes["@mozilla.org/network/io-service;1"]
									  .getService(Components.interfaces.nsIIOService);
 
       	var cookieService = Components.classes["@mozilla.org/cookieService;1"]
									  .getService(Components.interfaces.nsICookieService);  	
		
		if( ioService == null || cookieManager == null )
			return;
 
		var cookieURI = ioService.newURI( uri, null, null);

	   	cookieService.setCookieString( cookieURI, null, cookieString , null);		
	},

	// -----------------------------------------------------------
	// Boot error management
	// -----------------------------------------------------------
		
	setBootError: function( errorMsg )
	{
		$feedly( "[boot][error]" + errorMsg );
		
		this.bootError = errorMsg;
		this.fire( "onBootError", bootError );	
	},
	
	getBootError : function()
	{
		return this.bootError;
	},

	isCoreLoaded : function()
	{
		return typeof streets != "undefined";
	},

	lookupCore : function()
	{
		if( typeof streets == "undefined" )
		{
			$feedly( "[core][bug] someone is looking up core prematurely." );
			return null;			
		}
		else
		{
			return streets;
		}
	},

	/// OBSERVER INTERFACE
	
	registerObserver: function( obsObj )
	{
		if (this.fObservers.indexOf(obsObj) >= 0) {
			return 
		}
		// BDH: that.$feedly( "Registering observer for:" + id + " has " + fObservers.length + " observers. Hint: " +  obsObj.id );
	    this.fObservers.push(obsObj)
	}, 
	
	unregisterObserver : function( obsObj )
	{
		///D that.$feedly( "unregistering observer for:" +  that.id + " -- channel: " + getChannelId(obsObj) );
		var idx = this.fObservers.indexOf(obsObj)
		if (idx < 0) {
			return
		}
		this.fObservers.splice(idx,1)
	},
	
	removeAllObservers : function () 
	{
	    this.fObservers = []
	},

	fire : function ( aspect,a1,a2,a3,a4,a5,a6,a7,a8 )   // , arg1 , ..., argN
	{
	    // Are there any observers?
	    if( this.fObservers.length == 0 ) {
	        return
	    }
	    // copy list of observers, so we don't mutate while we loop
	    var observers = new Array()
	    for(var oI in this.fObservers) {
	        observers.push ( this.fObservers [oI] )
	    }

	    // The observer list may change as it fire notifications. Make sure that
	    // we take this into account
	    // Iterate through list of observers
	    for( var i=0;  i < observers.length; i++ ) 
		{
			try
			{
		        // Notify each observer
		        var obs = observers[i]
				
				if (this.fObservers.indexOf(obs) < 0) {
					continue
				}
		     
		        // 1. check if the aspect is a property of observer
		        if( obs[aspect] != null ) 
				{
		            // Call this method; here we don't pass aspect
		            obs[aspect].call (obs,a1,a2,a3,a4,a5,a6,a7,a8);
		            continue;
		        }  
		
		        // 2. check generic onEvent method
		        if (obs.onEvent != null) 
				{
		            // generic aspect: 'onChanged'
		            obs.onChanged.call ( obs, aspect,a1,a2,a3,a4,a5,a6,a7,a8 )
		            continue;
		        }
		        // 3. It's a function 
		        if ( typeof( obs ) == "function" ) 
				{
			        obs.call ( obs, aspect,a1,a2,a3,a4,a5,a6,a7,a8 )
			        continue;
			    }
			}
			catch( e )
			{
				// ignore
				that.$feedly( devhd.utils.ExceptionUtils.formatError( "notify observer", e ) + ". Hint about observer:" + observers[ i ].id + "." + aspect );
				// that.unregisterObserver( observers[i] );
			}
	    }
	    // done
	}
};

/**
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
*/
if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory( [ FeedlyBootComponent ] );
else
    var NSGetModule = XPCOMUtils.generateNSGetModule( [ FeedlyBootComponent ] );
